@@ -4,7 +4,7 @@
 # By: Cyril Lacoux <clacoux@easter-eggs.com>
 #     Valéry Febvre <vfebvre@easter-eggs.com>
 #
-# Copyright (C) 2011-2021 Cyril Lacoux, Easter-eggs
+# Copyright (C) 2011-2022 Cyril Lacoux, Easter-eggs
 # https://gitlab.com/yack/pyramid-helpers
 #
 # This file is part of pyramid-helpers.
@@ -37,7 +37,7 @@ from pyramid.static import static_view
 
 log = logging.getLogger(__name__)
 
-DECORATOR_RE = re.compile(r'^@(?P<func>paginate|validate)\((?P<args>.*)?\)$')
+DECORATOR_RE = re.compile(r'^@(?P<func>paginate|validate)\((?P<args>.+)?\)$')
 
 PAGER_PARAMS = {
     'sort': dict(description='Sorting key', type='UnicodeString'),
@@ -95,7 +95,7 @@ def inspect_schema_doc(tree, parameters, parent=None):
 
         name = line[1].split(' ')[-1].strip()
         if parent is not None:
-            name = '{0}.{1}'.format(parent, name)
+            name = f'{parent}.{name}'
         if name in parameters:
             parameters[name]['description'] = line[2].strip()
 
@@ -204,7 +204,7 @@ def api_doc(request, renderer_values):
     introspector = registry.introspector
 
     # Get defined routes
-    services = dict()
+    services = {}
 
     for item in introspector.get_category('routes'):
         introspectable = item['introspectable']
@@ -236,22 +236,23 @@ def api_doc(request, renderer_values):
             else:
                 continue
 
-            param_data = dict(
-                type=param_type,
-                parameter_type='path',
-                required=True,
-                default=None,
-                description=None,
-            )
-
             for name in predicate.names:
-                custom_predicates.append(name)
-                param_data['name'] = name
+                param_data = dict(
+                    name=name,
+                    type=param_type,
+                    parameter_type='path',
+                    required=True,
+                    default=None,
+                    description=None,
+                )
+
                 if predicate.__class__.__name__ == 'EnumPredicate':
                     param_data['values'] = predicate.params[name]
 
                 services[route_name]['parameters'].append(param_data)
 
+                custom_predicates.append(name)
+
         # get other predicates (not custom) present in pattern
         for name in route.match(route_intr['pattern']).keys():
             if name not in custom_predicates:
@@ -277,7 +278,7 @@ def api_doc(request, renderer_values):
                 services[route_name]['allowed'] = bool(request.has_permission(permission))
 
     # Get associated views
-    modules = dict()
+    modules = {}
 
     for item in introspector.get_category('views'):
         introspectable = item['introspectable']
@@ -323,8 +324,8 @@ def api_doc(request, renderer_values):
             decorator_name, params_str = match.groups()
 
             def __extract__(*args, **kwargs):
-                __extract__.args = args
-                __extract__.kwargs = kwargs
+                __extract__.args = args         # pylint: disable=cell-var-from-loop
+                __extract__.kwargs = kwargs     # pylint: disable=cell-var-from-loop
 
             globals_ = __globals__.copy()
             globals_['__extract__'] = __extract__
@@ -335,8 +336,8 @@ def api_doc(request, renderer_values):
             if decorator_name == 'paginate':
                 pager_name = __extract__.args[0]
 
-                for param_name in PAGER_PARAMS:
-                    param_data = PAGER_PARAMS[param_name].copy()
+                for param_name, param_data in PAGER_PARAMS.items():
+                    param_data = param_data.copy()
 
                     param_data.update(dict(
                         name=f'{pager_name}.{param_name}',
@@ -357,12 +358,12 @@ def api_doc(request, renderer_values):
             elif decorator_name == 'validate':
                 schema = __extract__.args[1]
 
-                parameters = dict()
+                parameters = {}
                 for name, field in schema.fields.items():
                     class_name = field.__class__.__name__
                     if class_name.endswith('Schema'):
                         for sub_name, sub_field in field.__dict__['fields'].items():
-                            parameter_name = '{0}.{1}'.format(name, sub_name)
+                            parameter_name = f'{name}.{sub_name}'
                             parameters[parameter_name] = inspect_validator(sub_field, parameter_name)
 
                         inspect_schema_doc((field.__class__,), parameters, name)
@@ -389,10 +390,10 @@ def api_doc(request, renderer_values):
     bootstrap_version = bootstrap_version.split('.')[0]
     if bootstrap_version == '3':
         required_libraries = ('bootstrap-datetimepicker', 'bootstrap-touchspin', 'font-awesome', 'handlebars', 'moment')
-    elif bootstrap_version == '4':
+    elif bootstrap_version in ('4', '5'):
         required_libraries = ('bootstrap-touchspin', 'font-awesome', 'handlebars', 'moment', 'tempusdominus')
     else:
-        raise HTTPBadRequest(detail=translate('Unsupported Bootstrap version: {0}'.format(bootstrap_version)))
+        raise HTTPBadRequest(detail=translate('Unsupported Bootstrap version: {0}').format(bootstrap_version))
 
     # Get libraries
     options = dict(
@@ -413,14 +414,14 @@ def api_doc(request, renderer_values):
 
     # Embbed CSS and JS files
     view = static_view('pyramid_helpers:static')
-    with open(view.find_resource_path('static/css/api-doc-bs{0}.css'.format(bootstrap_version)), 'r') as fp:
+    with open(view.find_resource_path(f'static/css/api-doc-bs{bootstrap_version}.css'), 'r', encoding='utf-8') as fp:
         renderer_values['css'] = fp.read()
 
-    with open(view.find_resource_path('static/js/api-doc-bs{0}.js'.format(bootstrap_version)), 'r') as fp:
+    with open(view.find_resource_path('static/js/api-doc.js'), 'r', encoding='utf-8') as fp:
         renderer_values['js'] = fp.read()
 
     # Set template according to Bootstrap version
-    request.override_renderer = '/api-doc-bs{0}.mako'.format(bootstrap_version)
+    request.override_renderer = f'/api-doc-bs{bootstrap_version}.mako'
 
     # Update renderer values
     renderer_values.update(dict(
