@@ -4,7 +4,7 @@
 # By: Cyril Lacoux <clacoux@easter-eggs.com>
 #     Valéry Febvre <vfebvre@easter-eggs.com>
 #
-# Copyright (C) 2011-2021 Cyril Lacoux, Easter-eggs
+# Copyright (C) 2011-2022 Cyril Lacoux, Easter-eggs
 # https://gitlab.com/yack/pyramid-helpers
 #
 # This file is part of pyramid-helpers.
@@ -35,7 +35,6 @@ import hashlib
 import logging
 
 from beaker.util import NoneType
-from beaker.util import verify_rules
 
 from pyramid.authentication import AuthTktCookieHelper
 from pyramid.authentication import extract_http_basic_credentials
@@ -45,6 +44,7 @@ from pyramid.authorization import Everyone
 
 from pyramid_helpers.utils import ConfigurationError
 from pyramid_helpers.utils import get_settings
+from pyramid_helpers.utils import parse_settings
 from pyramid_helpers.utils import request_cache
 from pyramid_helpers.utils import resolve_dotted
 
@@ -102,7 +102,7 @@ SETTINGS_RULES = {
     'policy:token': [
         ('header', (str, NoneType), 'header must be a string designating a HTTP header'),
         ('query_param', (str, NoneType), 'query_param must be a string designating a query parameter'),
-        ('get_username_by_token', (str, NoneType), 'get_user_by_token must be a string designating a valid callback'),
+        ('get_username_by_token', (str, NoneType), 'get_username_by_token must be a string designating a valid callback'),
     ],
 }
 
@@ -170,7 +170,7 @@ class MultiSecurityPolicy:
 
         for policy in policies:
             if policy not in self.POLICIES:
-                raise ConfigurationError('[AUTH] Invalid policy: {0}'.format(policy))
+                raise ConfigurationError(f'[AUTH] Invalid policy: {policy}')
 
         self.cookie = AuthTktCookieHelper(secret, hashalg=hashalg) if 'ticket' in policies else None
         self.policies = policies
@@ -190,7 +190,7 @@ class MultiSecurityPolicy:
 
         if request.authentication_policy == 'basic':
             # Ask client to re-authenticate
-            return [('WWW-Authenticate', 'Basic realm="%s"' % self.realm)]
+            return [('WWW-Authenticate', f'Basic realm="{self.realm}"')]
 
         if self.cookie:
             # Delete cookie
@@ -381,7 +381,7 @@ def get_user_by_username(username, request):
 def get_username_by_token(token, request):
     """ Wrapper to get_username_by_token function defined in settings """
 
-    params = get_settings(request, 'auth', 'auth')
+    params = get_settings(request, 'auth', 'policy:token')
     func = params['get_username_by_token']
 
     return func(token, request)
@@ -407,63 +407,6 @@ def on_new_request(event):
         request.authentication_policy = None
 
 
-def parse_settings(settings, include_defaults=True):
-    """ Parse and check authentication settings """
-
-    if include_defaults:
-        for section in SETTINGS_DEFAULTS:
-            if section not in settings:
-                settings[section] = SETTINGS_DEFAULTS[section].copy()
-                continue
-
-            for option, value in SETTINGS_DEFAULTS[section].items():
-                settings[section].setdefault(option, value)
-
-    for section, rules in SETTINGS_RULES.items():
-        if section in settings:
-            verify_rules(settings[section], rules)
-
-    # Check policies
-    if 'all' in settings['auth']['policies']:
-        settings['auth']['policies'] = MultiSecurityPolicy.POLICIES[:]
-    else:
-        for policy in settings['auth']['policies']:
-            if policy not in MultiSecurityPolicy.POLICIES:
-                raise ConfigurationError('[AUTH] Invalid policy for parameters policies in section [auth]: {0}'.format(policy))
-
-    # Check mode
-    if settings['auth']['mode'] is None:
-        raise ConfigurationError('[AUTH] Missing value for parameter mode in section [auth]')
-
-    if settings['auth']['mode'] not in AUTH_CLIENTS:
-        raise ConfigurationError('[AUTH] Invalid value for parameter mode in section [auth]: {0}'.format(settings['auth']['mode']))
-
-    # Check hashalg
-    if settings['policy:ticket']['hashalg'] not in hashlib.algorithms_available:
-        raise ConfigurationError('[AUTH] Invalid value for parameter hashalg in section [policy:ticket]: {0}'.format(settings['auth']['policy:ticket']['hashalg']))
-
-    # Resolve get_principals
-    func = resolve_dotted(settings['auth']['get_principals'])
-    if func is None:
-        raise ConfigurationError('[AUTH] Invalid value for parameter get_principals in section [auth]: {0}'.format(settings['auth']['get_principals']))
-    settings['auth']['get_principals'] = func
-
-    # Resolve get_user_by_username
-    func = resolve_dotted(settings['auth']['get_user_by_username'])
-    if func is None:
-        raise ConfigurationError('[AUTH] Invalid value for parameter get_user_by_username in section [auth]: {0}'.format(settings['auth']['get_user_by_username']))
-    settings['auth']['get_user_by_username'] = func
-
-    # Resolve get_username_by_token if needed
-    if 'token' in settings['auth']['policies']:
-        func = resolve_dotted(settings['policy:token']['get_username_by_token'])
-        if func is None:
-            raise ConfigurationError('[AUTH] Invalid value for parameter get_username_by_token in section [policy:token]: {0}'.format(settings['auth']['get_username_by_token']))
-        settings['policy:token']['get_username_by_token'] = func
-
-    return settings
-
-
 def includeme(config):
     """
     Set up standard configurator registrations. Use via:
@@ -482,7 +425,45 @@ def includeme(config):
     if settings is None:
         raise ConfigurationError('[AUTH] Invalid or missing configuration for AUTH, please check auth.filepath directive')
 
-    settings = parse_settings(settings)
+    settings = parse_settings(settings, SETTINGS_RULES, defaults=SETTINGS_DEFAULTS)
+
+    # Check policies
+    if 'all' in settings['auth']['policies']:
+        settings['auth']['policies'] = MultiSecurityPolicy.POLICIES[:]
+    else:
+        for policy in settings['auth']['policies']:
+            if policy not in MultiSecurityPolicy.POLICIES:
+                raise ConfigurationError(f'[AUTH] Invalid policy for parameters policies in section [auth]: {policy}')
+
+    # Check mode
+    if settings['auth']['mode'] is None:
+        raise ConfigurationError('[AUTH] Missing value for parameter mode in section [auth]')
+
+    if settings['auth']['mode'] not in AUTH_CLIENTS:
+        raise ConfigurationError(f'[AUTH] Invalid value for parameter mode in section [auth]: {settings["auth"]["mode"]}')
+
+    # Check hashalg
+    if settings['policy:ticket']['hashalg'] not in hashlib.algorithms_available:
+        raise ConfigurationError(f'[AUTH] Invalid value for parameter hashalg in section [policy:ticket]: {settings["auth"]["policy:ticket"]["hashalg"]}')
+
+    # Resolve get_principals
+    func = resolve_dotted(settings['auth']['get_principals'])
+    if func is None:
+        raise ConfigurationError(f'[AUTH] Invalid value for parameter get_principals in section [auth]: {settings["auth"]["get_principals"]}')
+    settings['auth']['get_principals'] = func
+
+    # Resolve get_user_by_username
+    func = resolve_dotted(settings['auth']['get_user_by_username'])
+    if func is None:
+        raise ConfigurationError(f'[AUTH] Invalid value for parameter get_user_by_username in section [auth]: {settings["auth"]["get_user_by_username"]}')
+    settings['auth']['get_user_by_username'] = func
+
+    # Resolve get_username_by_token if needed
+    if 'token' in settings['auth']['policies']:
+        func = resolve_dotted(settings['policy:token']['get_username_by_token'])
+        if func is None:
+            raise ConfigurationError(f'[AUTH] Invalid value for parameter get_username_by_token in section [policy:token]: {settings["policy:token"]["get_username_by_token"]}')
+        settings['policy:token']['get_username_by_token'] = func
 
     # Add fake user tween if needed
     if 'remote' in settings['auth']['policies']:
